from OpenGL.GL import *
from OpenGL.GLUT import *
from OpenGL.GLU import *
from OpenGL.GL.ARB.texture_cube_map import *
import sys
from Numeric import *
from LinearAlgebra import *
from PyICgFXEffect import *
from Pynv_dds import *
from types import *
from time import *

		

class Effect:
	
	parametersToSetAtBeginDict = {}
	parameterDict = {}
	
	
	def __init__(self, filename= 'default.fx', **kwords):
		self.effect = PyICgFXEffect()
		#self.parameterList = []
		self.parameterDict = {}
		self.matWorld = identity(4,typecode=Float)

		errors = self.effect.PyCgFXCreateEffectFromFileA(filename, 0)
		print errors
		
		self.effect.PyCgFXSetDevice('OpenGL',0)
		
		effectDesc = self.effect.GetDesc()
		
		for i in range(0,effectDesc.Parameters):
			pDesc =  self.effect.GetParameterDesc(i)
			#self.parameterList.append( pDesc )
			if pDesc.Semantic in [ 'View','WorldIT', 'World', 'ViewIT', 'WorldViewProjection', 'Time', 'Position']:
				self.parametersToSetAtBeginDict[pDesc.Name] = pDesc
			self.parameterDict[ pDesc.Name ] = pDesc
		
		self.effect.SetTechnique(0)
		self.effect.Validate()
		
		self.ddsimage = CDDSImage()
		
		for attr,val in kwords.items():
			setattr(self,attr,val)	
			
			
	def __getatt__( self, attr):
		
		pDesc = self.parameterDict[attr]
		if pDesc.Type == CgFXPT_FLOAT:
			return self.effect.GetMatrix(pDesc.Index, 4, 4)	
			
	
	def __setattr__( self, attr, val ):
		if not self.parameterDict.has_key(attr):
			self.__dict__[attr] = val
			return 
			
		pDesc = self.parameterDict[attr] 
		
		if pDesc.Type == CgFXPT_FLOAT:
			if self.parametersToSetAtBeginDict.has_key(attr):
				del self.parametersToSetAtBeginDict[attr]
			
			self.effect.SetMatrix(pDesc.Index, val, 4, 4)
					
		elif pDesc.Type == CgFXPT_TEXTURE:
			
			#annotationDesc = self.effect.GetAnnotationDesc(pDesc.Index, 0) 
			#filename = annotationDesc.Value
			#print filename
			texture = glGenTextures(1)
			self.ddsimage.load(val)
			
			if self.ddsimage.is_cubemap():
				glBindTexture(GL_TEXTURE_CUBE_MAP_ARB, texture)
				self.ddsimage.upload_textureCubemap()
			elif self.ddsimage.is_volume():
				glBindTexture(GL_TEXTURE_3D, texture)
				self.ddsimage.upload_texture3D()
			elif self.ddsimage.get_height() == 1:
				self.ddsimage.upload_texture1D()
			else:
				glBindTexture(GL_TEXTURE_2D, texture)
				self.ddsimage.upload_texture2D()
			
			self.effect.SetTexture(pDesc.Index, texture)
			
						 			
	def Begin(self):
		for p in self.parametersToSetAtBeginDict.values():
				
			matView = glGetFloatv(GL_MODELVIEW_MATRIX)
			matProjection = glGetFloatv(GL_PROJECTION_MATRIX)	
				
			if p.Semantic == 'World':
				transp = transpose( self.matWorld )
				self.effect.SetMatrix(p.Index,  transp, 4, 4)
			
			if p.Semantic == 'WorldIT':
				inv = inverse(self.matWorld)
				self.effect.SetMatrix(p.Index, inv, 4, 4)
				
			if p.Semantic == 'WorldViewProjection':
				wv = matrixmultiply( self.matWorld,matView )
				wvp = matrixmultiply(wv, matProjection )
				transposewvp = transpose(wvp)
				self.effect.SetMatrix(p.Index,  transposewvp, 4, 4)
				
			if p.Semantic == 'ViewIT':
				inv = inverse(matView)
				self.effect.SetMatrix(p.Index,  inv, 4, 4)

			if p.Semantic == 'View':
				inv = transpose(matView)
				self.effect.SetMatrix(p.Index,  inv, 4, 4)
				
			if p.Semantic == 'WorldViewIT':
				wv = matrixmultiply( self.matWorld,matView )
				inv = inverse(wv)
				self.effect.SetMatrix(p.Index,  inv, 4, 4)

			if p.Semantic == 'Time':
                                self.effect.SetFloat(p.Index, clock())

                        #if p.Semantic == 'Position':
                         #       self.effect.SetVector(p.Index, glGetLightfv(GL_LIGHT0,GL_POSITION),4)
		
		return self.effect.Begin(0)
		
	def Pass(self, p):
		self.effect.Pass(p)

	def End(self):
		self.effect.End()	
