#include "PyICgFXEffect.h"

PyICgFXEffect::PyICgFXEffect()
{
}

PyICgFXEffect::~PyICgFXEffect(void)
{
	m_pICgFXEffect->Release();
}

const char* PyICgFXEffect::PyCgFXCreateEffectFromFileA(
        char*				pSrcFile,
        DWORD				Flags)
{
	const char* errors;
	CgFXCreateEffectFromFileA((LPCSTR)pSrcFile, Flags, &m_pICgFXEffect, &errors);
	return errors;
}

const char* PyICgFXEffect::PyCgFXCreateEffect(
        char*               pSrcData,
        DWORD	            Flags)
{
	const char* errors;
	CgFXCreateEffect((LPCSTR)pSrcData, Flags, &m_pICgFXEffect, &errors);
	return errors;
}

const char* PyICgFXEffect::PyCgFXCreateEffectCompiler(
        char*               pSrcData,
        DWORD				Flags)
{

	const char* errors;
	CgFXCreateEffectCompiler((LPCSTR)pSrcData, Flags, &m_pICgFXEffectCompiler, &errors);
	return errors;
}

const char* PyICgFXEffect::PyCgFXCreateEffectCompilerFromFileA(
        char*				pSrcFile,
        DWORD				Flags)
{
	const char* errors;
	CgFXCreateEffectCompilerFromFileA((LPCSTR)pSrcFile, Flags, &m_pICgFXEffectCompiler, &errors);
	return errors;
}


const char* PyICgFXEffect::CompileEffect(DWORD Flags)
{
	const char* errors;
	m_pICgFXEffectCompiler->CompileEffect(Flags, &m_pICgFXEffect, &errors);
	return errors;
}


void PyICgFXEffect::PyCgFXSetDevice(const char* pDeviceName, DWORD pDevice)
{
	CgFXSetDevice(pDeviceName, (LPVOID)pDevice);
}

void PyICgFXEffect::PyCgFXFreeDevice(const char* pDeviceName, DWORD pDevice)
{
	CgFXFreeDevice(pDeviceName, (LPVOID)pDevice);
}

const char* PyICgFXEffect::PyCgFXGetErrors()
{
	const char* errors;
	CgFXGetErrors(&errors);
	return errors;
}
void PyICgFXEffect::PyCgFXRelease()
{
	CgFXRelease();
}


void PyICgFXEffect::SetTechnique(int pTechnique)
{
	m_pICgFXEffect->SetTechnique((LPCSTR)pTechnique);
}

char* PyICgFXEffect::GetTechnique()
{
	LPCSTR t;
	m_pICgFXEffect->GetTechnique(&t);
	return (char*)t;
}

bool PyICgFXEffect::Validate()
{
	return m_pICgFXEffect->Validate();
}

int PyICgFXEffect::Begin(DWORD Flags)
{
	UINT p;
	m_pICgFXEffect->Begin(&p, Flags);
	return (int)p;
	
}

void PyICgFXEffect::Pass(int passNum)
{
	m_pICgFXEffect->Pass(passNum);
}

void PyICgFXEffect::End()
{
	m_pICgFXEffect->End();
}

void PyICgFXEffect::OnLostDevice()
{
	m_pICgFXEffect->OnLostDevice();
}
void PyICgFXEffect::OnResetDevice()
{
	m_pICgFXEffect->OnResetDevice();
}

PyCgFXEFFECT_DESC PyICgFXEffect::GetDesc()
{
	CgFXEFFECT_DESC desc;
	PyCgFXEFFECT_DESC pydesc;
	m_pICgFXEffect->GetDesc(&desc);

	pydesc.Functions	= desc.Functions;
	pydesc.Parameters	= desc.Parameters;
	pydesc.Techniques	= desc.Techniques;

	return pydesc;
}

PyCgFXPARAMETER_DESC PyICgFXEffect::GetParameterDesc(int pParameter)
{
	CgFXPARAMETER_DESC desc;
	PyCgFXPARAMETER_DESC pydesc;
	m_pICgFXEffect->GetParameterDesc((LPCSTR)pParameter, &desc);

	pydesc.Annotations	= desc.Annotations;
	pydesc.Bytes		= desc.Bytes;
	for ( size_t i = 0 ; i < CGFX_MAX_DIMENSIONS; i++ ) 
	{
		pydesc.Dimension[i] = desc.Dimension[i];
		//printf("bla %d\n",pydesc.Dimension[i]);
	}
	pydesc.Index		= (UINT)desc.Index;
	pydesc.Name			= desc.Name;
	pydesc.Semantic		= desc.Semantic;
	pydesc.Type			= desc.Type;

	//printf("Index: %s \n",pydesc.Index);
	return pydesc;
}

PyCgFXANNOTATION_DESC PyICgFXEffect::GetAnnotationDesc(int pParameter, int pAnnotation)
{
	CgFXANNOTATION_DESC desc;
	PyCgFXANNOTATION_DESC pydesc;
	m_pICgFXEffect->GetAnnotationDesc((LPCSTR)pParameter, (LPCSTR)pAnnotation, &desc);

	pydesc.Bytes		= desc.Bytes;
	for ( size_t i = 0 ; i < CGFX_MAX_DIMENSIONS; i++ ) 
		pydesc.Dimension[i] = desc.Dimension[i];
	pydesc.Index		= (UINT)desc.Index;
	pydesc.Name			= desc.Name;
	pydesc.Type			= desc.Type;
	pydesc.Value		= (LPCSTR)desc.Value;

	return pydesc;
}

PyCgFXTECHNIQUE_DESC PyICgFXEffect::GetTechniqueDesc(int pTechnique)
{
	CgFXTECHNIQUE_DESC desc;
	PyCgFXTECHNIQUE_DESC pydesc;
	m_pICgFXEffect->GetTechniqueDesc((LPCSTR)pTechnique, &desc);

	pydesc.Index		= (UINT)desc.Index;
	pydesc.Name			= desc.Name;
	pydesc.Passes		= desc.Passes;
	pydesc.Properties	= desc.Properties;

	return pydesc;
}

PyCgFXPASS_DESC PyICgFXEffect::GetPassDesc(int pTechnique, int pPass)
{
	CgFXPASS_DESC desc;
	PyCgFXPASS_DESC pydesc;
	m_pICgFXEffect->GetPassDesc((LPCSTR)pTechnique, (LPCSTR)pPass, &desc);

	pydesc.Index		= (UINT)desc.Index;
	pydesc.Name			= desc.Name;

	return pydesc;
}

ULONG PyICgFXEffect::AddRef()
{
	return m_pICgFXEffect->AddRef();
}

ULONG PyICgFXEffect::Release()
{
	return m_pICgFXEffect->Release();
}

void PyICgFXEffect::SetFloat(int pName, FLOAT f)
{
	m_pICgFXEffect->SetFloat((LPCSTR)pName, f);
}

float PyICgFXEffect::GetFloat(int pName)
{
	float f;
	m_pICgFXEffect->GetFloat((LPCSTR)pName, &f);
	return f;
}

void PyICgFXEffect::SetVector(UINT pName, const float* pVector, UINT vecSize)
{
	//only implemented for length 4!

	float vector[4];

	vector[0] = *(pVector + 0);
	vector[1] = *(pVector + 1);
	vector[2] = *(pVector + 2);
	vector[3] = *(pVector + 3);

	m_pICgFXEffect->SetVector((LPCSTR)pName, &vector[0], vecSize);

}
	//virtual HRESULT GetVector(LPCSTR pName, float* pVector, UINT* vecSize) = 0;


void PyICgFXEffect::SetMatrix(UINT pName, const float* pMatrix, UINT nRows, UINT nCols)
{

	//only implemented for 4x4 case!

	float matrix[4][4];

	matrix[0][0] = *(pMatrix + 0);
	matrix[0][1] = *(pMatrix + 1);
	matrix[0][2] = *(pMatrix + 2);
	matrix[0][3] = *(pMatrix + 3);
	matrix[1][0] = *(pMatrix + 4);
	matrix[1][1] = *(pMatrix + 5);
	matrix[1][2] = *(pMatrix + 6);
	matrix[1][3] = *(pMatrix + 7);
	matrix[2][0] = *(pMatrix + 8);
	matrix[2][1] = *(pMatrix + 9);
	matrix[2][2] = *(pMatrix + 10);
	matrix[2][3] = *(pMatrix + 11);
	matrix[3][0] = *(pMatrix + 12);
	matrix[3][1] = *(pMatrix + 13);
	matrix[3][2] = *(pMatrix + 14);
	matrix[3][3] = *(pMatrix + 15);


	HRESULT hr = m_pICgFXEffect->SetMatrix((LPCSTR)pName, &matrix[0][0], nRows, nCols);
}

	
void  PyICgFXEffect::SetDword(int pName, DWORD dw)
{
	m_pICgFXEffect->SetDword((LPCSTR)pName, dw);
}

DWORD PyICgFXEffect::GetDword(int pName)
{
	DWORD dw;
	m_pICgFXEffect->GetDword((LPCSTR)pName, &dw);
	return dw;
}

void  PyICgFXEffect::SetBoolValue(int pName, bool bvalue)
{
	m_pICgFXEffect->SetBoolValue((LPCSTR)pName, bvalue);
}

bool  PyICgFXEffect::GetBoolValue(int pName)
{
	bool b;
	m_pICgFXEffect->GetBoolValue((LPCSTR)pName, &b);
	return b;
}

void  PyICgFXEffect::SetString(int pName, LPCSTR pString)
{
	m_pICgFXEffect->SetString((LPCSTR)pName, pString);
}

char* PyICgFXEffect::GetString(int pName)
{
	LPCSTR str;
	m_pICgFXEffect->GetString((LPCSTR)pName, &str);
	return (char*)str;
}
	
void PyICgFXEffect::SetTexture(int pName, DWORD textureHandle)
{
	m_pICgFXEffect->SetTexture((LPCSTR)pName, textureHandle);
}

DWORD PyICgFXEffect::GetTexture(int pName)
{
	DWORD h;
	m_pICgFXEffect->GetTexture((LPCSTR)pName, &h);
	return h;
}

void  PyICgFXEffect::SetVertexShader(int pName, DWORD vsHandle)
{
	m_pICgFXEffect->SetVertexShader((LPCSTR)pName, vsHandle);
}

DWORD PyICgFXEffect::GetVertexShader(int pName)
{
	DWORD h;
	m_pICgFXEffect->GetVertexShader((LPCSTR)pName, &h);
	return h;
}

void  PyICgFXEffect::SetPixelShader(int pName, DWORD psHandle)
{
	m_pICgFXEffect->SetPixelShader((LPCSTR)pName, psHandle);
}

DWORD PyICgFXEffect::GetPixelShader(int pName)
{
	DWORD h;
	m_pICgFXEffect->GetPixelShader((LPCSTR)pName, &h);
	return h;
}
