#include  <stdio.h>
#include "ICgFXEffect.h"
#include "Python.h"
#include "Numeric/arrayobject.h"




#define CgFXPT_UNKNOWN 0
#define	CgFXPT_BOOL 1
#define	CgFXPT_INT 2
#define CgFXPT_FLOAT 3
#define CgFXPT_DWORD 4
#define CgFXPT_STRING 5

    // Ref counted objects; Can be accessed as objects or as strings
#define CgFXPT_TEXTURE 6
#define CgFXPT_CUBETEXTURE 7
#define CgFXPT_VOLUMETEXTURE 8
#define CgFXPT_VERTEXSHADER 9
#define CgFXPT_PIXELSHADER 10

    // Complex type
#define CgFXPT_STRUCT 11
    
    // force 32-bit size enum
#define CgFXPT_FORCE_DWORD      = 0x7fffffff


class PyCgFXEFFECT_DESC
{
public:
	UINT Parameters;	// Number of parameters
	UINT Techniques;	// Number of techniques
	UINT Functions;		// Number of functions
};

class PyCgFXPARAMETER_DESC
{
public:
	LPCSTR Name;								// Parameter name.
	UINT Index;									// Parameter index 

	// Usage
	LPCSTR Semantic;							// Semantic meaning

	// Type
	UINT Type;						// Parameter type
	UINT Dimension[CGFX_MAX_DIMENSIONS];		// Elements in array
	UINT Bytes;									// Total size in bytes

	// Annotations
	UINT Annotations;							// Number of annotations.
};

class PyCgFXANNOTATION_DESC
{
public:
    LPCSTR Name;								// Annotation name
    UINT Index;									// Annotation index (cast to LPCSTR)
	LPCSTR Value;								// Ska egentligen vara LPCVOID 

    // Type
    UINT Type;						// Annotation type
    UINT Dimension[CGFX_MAX_DIMENSIONS];        // Elements in array
    UINT Bytes;									// Total size in bytes

};

class PyCgFXTECHNIQUE_DESC
{
public:
    LPCSTR Name;								// Technique name
    UINT Index;									// Technique index (cast to LPCSTR)

    UINT Properties;							// Number of properties
    UINT Passes;								// Number of passes

};

class PyCgFXPASS_DESC
{
public:
    LPCSTR Name;								// Pass name
    UINT Index;									// Pass index (cast to LPCSTR)

};



class PyICgFXEffect
{
public:
	PyICgFXEffect();
	~PyICgFXEffect(void);

	const char* PyCgFXCreateEffectFromFileA(
        char*				pSrcFile,
        DWORD				Flags);

	const char* PyCgFXCreateEffect(
        char*               pSrcData,
        DWORD	            Flags);

	const char* PyCgFXCreateEffectCompiler(
        char*               pSrcData,
        DWORD				Flags);

	const char* PyCgFXCreateEffectCompilerFromFileA(
        char*				pSrcFile,
        DWORD				Flags);

	const char* CompileEffect(DWORD Flags);

	void		PyCgFXSetDevice(const char* pDeviceName, DWORD pDevice);
	void		PyCgFXFreeDevice(const char* pDeviceName, DWORD pDevice);
	const char* PyCgFXGetErrors();
	void		PyCgFXRelease();

	void		SetTechnique(int pTechnique);
	char*		GetTechnique();

	// Validate current technique:
	bool		Validate();

	// Returns the number of passes in pPasses:
	int			Begin(DWORD Flags);
	void		Pass(int passNum);
	void		End();

	// Descriptions
	/* Return description of effect */
	PyCgFXEFFECT_DESC GetDesc();
	/* Return description of the named/indexed parameter */
	PyCgFXPARAMETER_DESC GetParameterDesc(int pParameter);
	/* Return description of the named/indexed annotation */
	PyCgFXANNOTATION_DESC GetAnnotationDesc(int pParameter, int pAnnotation);
	/* Return description of the named/indexed technique */
	PyCgFXTECHNIQUE_DESC GetTechniqueDesc(int pTechnique);
	/* Return description of pass for given technique */
	PyCgFXPASS_DESC GetPassDesc(int pTechnique, int pPass);

	ULONG AddRef();
	ULONG Release();

	// Get/Set Parameters
	
	void  SetFloat(int pName, float f);
	float GetFloat(int pName);
	
	void  SetVector(UINT pName, const float* pVector, UINT vecSize);
	
	void  SetMatrix(UINT pName, const float* pMatrix, UINT nRows, UINT nCols);
	
	void  SetDword(int pName, DWORD dw);
	DWORD GetDword(int pName);

	void  SetBoolValue(int pName, bool bvalue);
	bool  GetBoolValue(int pName);
	void  SetString(int pName, LPCSTR pString);
	char* GetString(int pName);
	void  SetTexture(int pName, DWORD textureHandle);
	DWORD GetTexture(int pName);

	void  SetVertexShader(int pName, DWORD vsHandle);
	DWORD GetVertexShader(int pName);

	void  SetPixelShader(int pName, DWORD psHandle);
	DWORD GetPixelShader(int pName);

	void OnLostDevice();
	void OnResetDevice();

private:
	ICgFXEffect*			m_pICgFXEffect;
	ICgFXEffectCompiler*	m_pICgFXEffectCompiler;
};


