/* File: PyICgFXEffect.i */

#include "typemaps.i"


%module PyICgFXEffect

// Convert a 4 element float tuple to a 4 element C array
%typemap(python,in) float[4] (float temp[4]) {
if (PyTuple_Check($source)) {
	if (!PyArg_ParseTuple($source,"ffff", temp, temp+1, temp+2, temp+3)) {
		PyErr_SetString(PyExc_TypeError,"expected a 4-element tuple of floats");
		return NULL;
	}
	$target = temp;
} else {
	PyErr_SetString(PyExc_TypeError,"expected a tuple.");
	return NULL;
	}
}

// Convert a 4 element float tuple to a 4 element C array
%typemap(python,in) const float* pVector (float temp[4]) {
if (PyTuple_Check($source)) {
	if (!PyArg_ParseTuple($source,"ffff", temp, temp+1, temp+2, temp+3)) {
		PyErr_SetString(PyExc_TypeError,"expected a 4-element tuple of floats");
		return NULL;
	}
	$target = temp;
} else {
	PyErr_SetString(PyExc_TypeError,"expected a tuple.");
	return NULL;
	}
}



	// Convert a matrix (list,list) of floats to a 2d C array 
//	%typemap(python,in) const float* pMatrix (float temp[16])
//	{
//	if (PyList_Check($source)) {
//		PyObject* val;
//		int i;
//		int size = PyList_Size($source);
//		for ( i = 0 ; i < size ; i++ ) {
//			val = PyList_GetItem($source,i);
//			temp[i] = PyFloat_AsDouble(val);
//		}
//		$target = temp;
//	}
//	else {	
//		PyErr_SetString(PyExc_TypeError, "expected a list.");
//		return NULL;
//	}
//	}

//Convert a matrix array of floats to a 2d C array
%typemap(python,in) const float* pMatrix (float temp[16])
{
	if (PyArray_Check($source)) 
	{	
		int i,j, dim, rows, cols;
		PyArrayObject* array;
		//array = (PyArrayObject*)$source;
		array = (PyArrayObject*)PyArray_ContiguousFromObject($source, PyArray_DOUBLE, 1,2);
		dim = array->nd;
		if (dim == 0)
		{
			PyErr_SetString(PyExc_TypeError, "array has dimension 0.");
			return NULL;
		}
		else if (dim == 1)
		{
			for (i = 0; i < array->dimensions[0]; i++)
			{
				temp[i] = (float)*(array->data + i * array->strides[0]);
			}
		}
		else if (dim == 2)
		{
			for (i = 0; i < array->dimensions[0]; i++)
			{ 
				for (j = 0; j < array->dimensions[1]; j++)
				{	
					temp[i * array->dimensions[0] +j] = *(double*)(array->data + i * array->strides[0] + j * array->strides[1]);
				}
			}	
		}
		else
		{
			PyErr_SetString(PyExc_TypeError, "too many dimensions in array.");
			return NULL;
		}	
		$target = temp;
	}
	else
	{
		PyErr_SetString(PyExc_TypeError, "expected an array.");
		return NULL;
	}
}
		 
		


%typemap(python,argout) double *OutValue 
{
	PyObject *o;
	o = PyFloat_FromDouble(*$source);
	if ((!$target) || ($target == Py_None)) 
	{
		$target = o;
	} 
	else 
	{
		if (!PyList_Check($target)) 
		{
			PyObject *o2 = $target;
			$target = PyList_New(0);
			PyList_Append($target,o2);
			Py_XDECREF(o2);
		}
		PyList_Append($target,o);
		Py_XDECREF(o);
	}
}
%typemap(python,ignore) double *OutValue(double temp) {
	$target = &temp; /* Assign the pointer to a local variable */
}

// This allows a C function to return a char ** as a Python list
%typemap(python,out) const char **OutValue{
int len,i;
len = 0;
while ($source[len]) len++;
$target = PyList_New(len);
for (i = 0; i < len; i++) {
PyList_SetItem($target,i,PyString_FromString($source[i]));
}
}


%{
#include "PyICgFXEffect.h"
%}

%init %{
	import_array();
%}

/* Let's just grab the original header file here */
%include "PyICgFXEffect.h"
%include "cgfx_stddefs.h"
